﻿-- Grid2LuaTexts is based on PitBull LuaTexts
-- All rights reserved by the authors of Pitbull4 (ckknight, Shefki)

local ScriptEnv = setmetatable({}, {__index = _G})
Grid2.ScriptEnv = ScriptEnv

local function GetShortNumber(v, setPlusSign)
	local sign
	if v < 0 then
		sign = "-"
		v = -v
	else
		sign = setPlusSign and "+" or ""
	end
	if v >= 1000 then
		v = ("%.1fk"):format(v / 1000)
	end
	return sign..v
end
ScriptEnv.GetShortNumber = GetShortNumber

local function Round(number, digits)
	if not digits then
		digits = 0
	end
	local mantissa = 10^digits
	local norm = number*mantissa + 0.5
	local norm_floor = math.floor(norm)
	if norm == norm_floor and (norm_floor % 2) == 1 then
		return (norm_floor-1) / mantissa
	else
		return norm_floor / mantissa
	end
end
ScriptEnv.Round = Round

local function Short(value,format)
	if type(value) == "number" then
		local fmt
		if value >= 10000000 or value <= -10000000 then
			fmt = "%.1fm"
			value = value / 1000000
		elseif value >= 1000000 or value <= -1000000 then
			fmt = "%.2fm"
			value = value / 1000000
		elseif value >= 100000 or value <= -100000 then
			fmt = "%.0fk"
			value = value / 1000
		elseif value >= 10000 or value <= -10000 then
			fmt = "%.1fk"
			value = value / 1000
		else
			fmt = "%d"
			value = math.floor(value+0.5)
		end
		if format then
			return fmt:format(value)
		else
			return fmt, value
		end
	else
		local fmt_a, fmt_b
		local a,b = value:match("^(%d+)/(%d+)$")
		if a then
			a, b = tonumber(a), tonumber(b)
			if a >= 10000000 or a <= -10000000 then
				fmt_a = "%.1fm"
				a = a / 1000000
			elseif a >= 1000000 or a <= -1000000 then
				fmt_a = "%.2fm"
				a = a / 1000000
			elseif a >= 100000 or a <= -100000 then
				fmt_a = "%.0fk"
				a = a / 1000
			elseif a >= 10000 or a <= -10000 then
				fmt_a = "%.1fk"
				a = a / 1000
			end
			if b >= 10000000 or b <= -10000000 then
				fmt_b = "%.1fm"
				b = b / 1000000
			elseif b >= 1000000 or b <= -1000000 then
				fmt_b = "%.2fm"
				b = b / 1000000
			elseif b >= 100000 or b <= -100000 then
				fmt_b = "%.0fk"
				b = b / 1000
			elseif b >= 10000 or b <= -10000 then
				fmt_b = "%.1fk"
				b = b / 1000
			end
			if format then
				return (fmt_a.."/"..fmt_b):format(a,b)
			else
				return fmt_a.."/"..fmt_b,a,b
			end
		else
			return value
		end
	end
end
ScriptEnv.Short = Short

local function VeryShort(value,format)
	if type(value) == "number" then
		local fmt
		if value >= 1000000 or value <= -1000000 then
			fmt = "%.0fm"
			value = value / 1000000
		elseif value >= 1000 or value <= -1000 then
			fmt = "%.0fk"
			value = value / 1000
		else
			fmt = "%.0f"
		end
		if format then
			return fmt:format(value)
		else
			return fmt, value
		end
	else
		local a,b = value:match("^(%d+)/(%d+)")
		if a then
			local fmt_a, fmt_b
			a, b = tonumber(a), tonumber(b)
			if b >= 1000000 or b <= -1000000 then
				fmt_b = "%.0fm"
				b = b / 1000000
			elseif b >= 1000 or b <= -1000 then
				fmt_b = "%.0fk"
				b = b / 1000
			end
			if a >= 1000000 or a <= -1000000 then
				fmt_a = "%.0fm"
				a = a / 1000000
			elseif a >= 1000 or a <= -1000 then
				fmt_a = "%.0fk"
				a = a / 1000
			end
			if format then
				return (fmt_a.."/"..fmt_b):format(a,b)
			else
				return fmt_a.."/"..fmt_b,a,b
			end
		else
			return value
		end
	end
end
ScriptEnv.VeryShort = VeryShort

local function Percent(x, y)
	if y ~= 0 then
		return Round(x / y * 100,1)
	else
		return 0
	end
end
ScriptEnv.Percent = Percent

local function HP(unit)
	local hp = UnitHealth(unit)
	if hp == 1 and UnitIsGhost(unit) then
		return 0
	end
	return hp
end
ScriptEnv.HP = HP

-- Just use the Blizzard API no change needed
-- only reason this is here is for symmetry,
-- it feels weird to have HP (which we need
-- to avoid the hp = 1 while dead crap), but
-- not have MaxHP
local MaxHP = UnitHealthMax
ScriptEnv.MaxHP = MaxHP

local luaTexts = {
	[L["Health"]] = {
		[L["Grid2"]] = {
			code = [[return GetShortNumber(UnitHealth(unit))]],
		},
		[L["Absolute"]] = {
			code = [[return ("%s/%s"):format(HP(unit),MaxHP(unit))]],
		},
		[L["Absolute short"]] = {
			code = [[return ("%s/%s"):format(Short(HP(unit),true),Short(MaxHP(unit),true))]],
		},
		[L["Difference"]] = {
			code = [[
				local miss = MaxHP(unit) - HP(unit) 
				if miss ~= 0 then
					return ("-%d"):format(miss)
				end]],
		},
		[L["Percent"]] = {
			code = [[return ("%s%%"):format(Percent(HP(unit),MaxHP(unit)))]],
		},
		[L["Mini"]] = {
			code = [[return VeryShort(HP(unit), true)]],
		},
		[L["Smart"]] = {
			code = [[
				local cur, max = HP(unit), MaxHP(unit)
				if UnitIsFriend(unit,"player") then
					local miss = max - cur 
					if miss ~= 0 then
						return ("|cffff7f7f%d|r"):format(miss)
					else
						return ""
					end
				else
					return ("%s/%s"):format(Short(cur,true),Short(max,true))
				end]],
		},
		[L["Absolute and percent"]] = {
			code = [[
				local cur, max = HP(unit), MaxHP(unit)
				return ("%s/%s || %s%%"):format(Short(cur,true),Short(max,true),Percent(cur,max))]],
		},
		[L["Informational"]] = {
			code = [[
				local cur, max = HP(unit), MaxHP(unit)
				if UnitIsFriend(unit,"player") then
					local miss = max - cur 
					if miss ~= 0 then
						return ("|cffff7f7f%s|r || %s/%s || %s%%"):format(Short(miss,true),Short(cur,true),Short(max,true),Percent(cur,max))
					end
				else
					return ("%s/%s"):format(Short(cur,true),Short(max,true))
				end]],
		},
	},
}
Grid2.luaTexts = luaTexts

local func_cache = {}

local handlerArray = {}
handlerArray[1] = "return function (self, unit) "
handlerArray[2] = ""
handlerArray[3] = " end"
local prev_MakeTextHandler = Grid2.MakeTextHandler
function Grid2:MakeTextHandler(object, ...)
	local dbx = object.dbx
	local base = dbx.base
	local entry = dbx.entry

	if (base and entry) then
		local code = luaTexts[base][entry].code
		handlerArray[2] = code

		local handler = table.concat(handlerArray)
		local func, err = loadstring(handler, "Grid2_LuaTexts:" .. object.name)
		if (func) then
			setfenv(func, Grid2.ScriptEnv)
			object.GetText = func()
		else
			geterrorhandler()(err)
			object.GetText = function (self, unit)
				return err
			end
		end
	else
		prev_MakeTextHandler(self, object, ...)
	end
end

local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	upgrade = DBL:LoadOptions("Grid2LuaTextsOptions", dblData, nil, "account", 1) or upgrade

	return upgrade
end

-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)

	local Grid2LuaTextsOptions = Grid2Options.plugins["Grid2LuaTextsOptions"]
	if (Grid2LuaTextsOptions) then
		flatten = DBL:UpgradeDefaults("Grid2LuaTextsOptions", dblData, Grid2LuaTextsOptions.UpgradeDefaults, "account", 1) or flatten
	end
	
	return flatten
end

